//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// ENBSeries Fallout4 effect file                                   //
// visit http://enbdev.com for updates                              //
// Author: Boris Vorontsov                                          //
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                                                                  //
//  TODIE enbeffect.fx by kingeric1992                              //
//                                                                  //
//  Featuring:                                                      //
//    independent Dawn, Sunrise, Day, Sunset, Dusk, Night prifiles  //
//              with Exterior/Interior seperation.                  //
//                                                                  //
//     Along with custom SplitTone ColorGrading function            //
//                                                                  //
//  For more info,  visit                                           //
//    http://enbseries.enbdev.com/forum/viewtopic.php?f=7&t=5330    //
//                                                                  //
//  update: Dec.03.2016                                             //
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

int    Title0 <string UIName="\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4\xA4"; float UIMin=0.0; float UIMax=0.0; > = { 0 };
float  ST_LowRange       < string UIName="SplitTone LowRange";   > = {0.3333};
float  ST_HighRange      < string UIName="SplitTone HighRange";  > = {0.6666};
int    ME <string UIName=" _(:3 \xB8/)_"; float UIMin=0.0; float UIMax=0.0; > = { 0 };

#define LUM_COEFF  float3(0.2125, 0.7154, 0.0721)   //709
#define DELTA      0.0000001

//+++++++++++++++++++++++++++++
//external enb parameters & shared resources, do not modify
//+++++++++++++++++++++++++++++
float4  Timer;              //x = generic timer in range 0..1, period of 16777216 ms (4.6 hours), y = average fps, w = frame time elapsed (in seconds)
float4  ScreenSize;         //x = Width, y = 1/Width, z = Width/Height, w = Height/Width,
float   AdaptiveQuality;    //changes in range 0..1, 0 means full quality, 1 lowest dynamic quality (0.33, 0.66 are limits for quality levels)
float4  Weather;            //x = current weather index, y = outgoing weather index, z = weather transition, w = time of the day in 24 standart hours. 
float4  TimeOfDay1;         //x = dawn, y = sunrise, z = day, w = sunset. Interpolators range from 0..1
float4  TimeOfDay2;         //x = dusk, y = night. Interpolators range from 0..1
float   ENightDayFactor;    //changes in range 0..1, 0 means that night time, 1 - day time
float   EInteriorFactor;    //changes 0 or 1. 0 means that exterior, 1 - interior
//keyboard controlled temporary variables. Press and hold key 1,2,3...8 together with PageUp or PageDown to modify. By default all set to 1.0
float4  tempF1, tempF2, tempF3; //1,2,3,4,5,6,7,8,9,0
//    0 = none      1 = left          2 = right         3 = left+right
//    4 = middle    5 = left+middle   6 = right+middle  7 = left+right+middle (or rather cat is sitting on your mouse)
float4	tempInfo1;          // xy = cursor position in range 0..1 of screen; z = is shader editor window active; w = mouse buttons with values 0..7 as aboves:
float4	tempInfo2;          // xy = cursor position of previous left click, zw = cursor position of previous right click
float4  Params01[6];        // FO4 parameters
float4  ENBParams01;        // x - bloom amount; y - lens amount

Texture2D TextureColor;     // hdr color
Texture2D TextureBloom;     // vanilla or enb bloom
Texture2D TextureLens;      // enb lens fx
Texture2D TextureDepth;     // scene depth
Texture2D TextureAdaptation;// vanilla or enb adaptation, .x only 
Texture2D TextureAperture;  // this frame aperture 1*1 R32F hdr red channel only. computed in depth of field shader file
Texture2D TextureNeutralLUT  <string ResourceName = "LUT_Base.bmp";>;

SamplerState        SamplerPoint
{
    Filter = MIN_MAG_MIP_POINT;//MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

SamplerState        SamplerLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

BlendState DualSrcAlpha
{
    BlendEnable[0]  =TRUE;
    SrcBlend        =SRC1_ALPHA;
    DestBlend       =ONE;
    BlendOp         =ADD;
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Functions
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "enbeffect_colorcorrection.fxh"

float3 Reinhard( float3 color, float Lwhite, float Curve) 
{
    return color / (max(color.r, max(color.g, color.b)) + Curve) * (1 + Curve / Lwhite);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Shaders
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

struct InputStruct
{
    float               PassWeight;
    float               Exposure, Lwhite, Curve;
    GameCCStruct        GameCCData; 
    ChannelMixerStruct  ChannelMixerData;
    LutStruct           LutData;
    ColorWheelStruct    ColorWheelData;     
    SplitToneStruct     SplitToneData;
};

float4 DrawFunc( float2 txcoord, InputStruct IN)
{
    float4 res   = TextureColor.Sample(SamplerPoint,  txcoord);
    float4 bloom = TextureBloom.Sample(SamplerLinear, txcoord);
    float4 lens  = TextureLens.Sample(SamplerLinear, txcoord);
    float3 color = (res.rgb + bloom.rgb * max( ENBParams01.x - res.rgb, 0.0)) * IN.Exposure;
    color = Reinhard(color, IN.Lwhite, IN.Curve); 
    color = lerp(color, Lut(          saturate(color), IN.LutData),          IN.LutData.Weight          * IN.LutData.State          );
    color = lerp(color, ColorWheel(   saturate(color), IN.ColorWheelData),   IN.ColorWheelData.Weight   * IN.ColorWheelData.State   ); 
    color = lerp(color, GameCC(       saturate(color), IN.GameCCData),       IN.GameCCData.Weight       * IN.GameCCData.State       );
    color = lerp(color, ChannelMixer( saturate(color), IN.ChannelMixerData), IN.ChannelMixerData.Weight * IN.ChannelMixerData.State );
    color = lerp(color, SplitTone(    saturate(color), IN.SplitToneData),    IN.SplitToneData.Weight    * IN.SplitToneData.State    );
    color = color * ( 1.0 - lens.a) + lens.rgb * ENBParams01.y; // equvilant to srcblend = one, destblend = invsrcalpha.
    return float4(color.rgb, IN.PassWeight);
}

void VS_Draw( inout float4 pos : SV_POSITION, inout float2 txcoord : TEXCOORD0 )
{
    pos.w = 1.0;
}

float4 PS_Draw( float4 pos : SV_POSITION, float2 txcoord : TEXCOORD0, uniform InputStruct ExtIN, uniform InputStruct IntIN, out float4 weight : SV_Target1) : SV_Target
{
    clip( ExtIN.PassWeight == 0.0? -1.0:1.0 );
    weight = ExtIN.PassWeight;
    if(EInteriorFactor) return DrawFunc(txcoord, IntIN);
    else                return DrawFunc(txcoord, ExtIN);
}

float4 PS_Blank() : SV_Target0 { return 0.0; }

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// UI profiles
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

static float MiddleGray = TextureAdaptation.Sample(SamplerPoint, 0.5).x;

#define TOSTRING(a) #a

Texture2D  TextureLUT_ExtDawn  <string ResourceName = "LUT_ExtDawn.bmp";>;
#define suffix ExtDawn
#define LUTTEXTURE TextureLUT_ExtDawn
#define PASSWEIGHT TimeOfDay1.x
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_ExtSunrise  <string ResourceName = "LUT_ExtSunrise.bmp";>;
#define suffix ExtSunrise
#define LUTTEXTURE TextureLUT_ExtSunrise
#define PASSWEIGHT TimeOfDay1.y
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_ExtDay  <string ResourceName = "LUT_ExtDay.bmp";>;
#define suffix ExtDay
#define LUTTEXTURE TextureLUT_ExtDay
#define PASSWEIGHT TimeOfDay1.z
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_ExtSunset  <string ResourceName = "LUT_ExtSunset.bmp";>;
#define suffix ExtSunset
#define LUTTEXTURE TextureLUT_ExtSunset
#define PASSWEIGHT TimeOfDay1.w
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_ExtDusk  <string ResourceName = "LUT_ExtDusk.bmp";>;
#define suffix ExtDusk
#define LUTTEXTURE TextureLUT_ExtDusk
#define PASSWEIGHT TimeOfDay2.x
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_ExtNight  <string ResourceName = "LUT_ExtNight.bmp";>;
#define suffix ExtNight
#define LUTTEXTURE TextureLUT_ExtNight
#define PASSWEIGHT TimeOfDay2.y
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_IntDawn  <string ResourceName = "LUT_IntDawn.bmp";>;
#define suffix IntDawn
#define LUTTEXTURE TextureLUT_IntDawn
#define PASSWEIGHT TimeOfDay1.x
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_IntSunrise  <string ResourceName = "LUT_IntSunrise.bmp";>;
#define suffix IntSunrise
#define LUTTEXTURE TextureLUT_IntSunrise
#define PASSWEIGHT TimeOfDay1.y
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_IntDay  <string ResourceName = "LUT_IntDay.bmp";>;
#define suffix IntDay
#define LUTTEXTURE TextureLUT_IntDay
#define PASSWEIGHT TimeOfDay1.z
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_IntSunset  <string ResourceName = "LUT_IntSunset.bmp";>;
#define suffix IntSunset
#define LUTTEXTURE TextureLUT_IntSunset
#define PASSWEIGHT TimeOfDay1.w
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_IntDusk  <string ResourceName = "LUT_IntDusk.bmp";>;
#define suffix IntDusk
#define LUTTEXTURE TextureLUT_IntDusk
#define PASSWEIGHT TimeOfDay2.x
#include "enbeffect_UI.fxh"

Texture2D  TextureLUT_IntNight  <string ResourceName = "LUT_IntNight.bmp";>;
#define suffix IntNight
#define LUTTEXTURE TextureLUT_IntNight
#define PASSWEIGHT TimeOfDay2.y
#include "enbeffect_UI.fxh"


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//techniques
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

technique11 Draw <string UIName="TODIE";>
{
    pass p0 //clear the buffer
    {
        SetVertexShader(CompileShader(vs_5_0, VS_Draw()));
        SetPixelShader( CompileShader(ps_5_0, PS_Blank()) );
    }
    pass p1
    {
        SetVertexShader(CompileShader(vs_5_0, VS_Draw()));
        SetPixelShader(CompileShader(ps_5_0, PS_Draw( UI_ExtDawn, UI_IntDawn )) );
        SetBlendState(DualSrcAlpha, float4(1.0, 1.0, 1.0, 1.0), 0xFFFFFFFF);
    }
    pass p2
    {
        SetVertexShader(CompileShader(vs_5_0, VS_Draw()));
        SetPixelShader(CompileShader(ps_5_0, PS_Draw( UI_ExtSunrise, UI_IntSunrise )) );
        SetBlendState(DualSrcAlpha, float4(1.0, 1.0, 1.0, 1.0), 0xFFFFFFFF);
    }
    pass p3
    {
        SetVertexShader(CompileShader(vs_5_0, VS_Draw()));
        SetPixelShader(CompileShader(ps_5_0, PS_Draw( UI_ExtDay, UI_IntDay )) );
        SetBlendState(DualSrcAlpha, float4(1.0, 1.0, 1.0, 1.0), 0xFFFFFFFF);
    }
    pass p4
    {
        SetVertexShader(CompileShader(vs_5_0, VS_Draw()));
        SetPixelShader(CompileShader(ps_5_0, PS_Draw( UI_ExtSunset, UI_IntSunset )) );
        SetBlendState(DualSrcAlpha, float4(1.0, 1.0, 1.0, 1.0), 0xFFFFFFFF);
    }
    pass p5
    {
        SetVertexShader(CompileShader(vs_5_0, VS_Draw()));
        SetPixelShader(CompileShader(ps_5_0, PS_Draw( UI_ExtDusk, UI_IntDusk )) );
        SetBlendState(DualSrcAlpha, float4(1.0, 1.0, 1.0, 1.0), 0xFFFFFFFF);
    }
    pass p6
    {
        SetVertexShader(CompileShader(vs_5_0, VS_Draw()));
        SetPixelShader(CompileShader(ps_5_0, PS_Draw( UI_ExtNight, UI_IntNight )) );
        SetBlendState(DualSrcAlpha, float4(1.0, 1.0, 1.0, 1.0), 0xFFFFFFFF);
    }
    
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Vanilla post process. Do not modify
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float4	PS_DrawOriginal( float4 pos : SV_POSITION, float2 txcoord : TEXCOORD0) : SV_Target
{
	float4	res;
	float4	color;
	color=TextureColor.Sample(SamplerPoint, txcoord.xy); //hdr scene color

	float4	r0, r1, r2, r3;
	r0.xyz = color.xyz;
	r1.xy = Params01[4].zw * txcoord.xy;
	r1.xyz = TextureBloom.Sample(SamplerLinear, r1.xy).xyz;
	r0.w = TextureAdaptation.Sample(SamplerPoint, txcoord.xy).x;
	r1.w = Params01[1].z / (0.001 + r0.w);
	r2.x = r1.w < Params01[1].y;
	r1.w = r2.x ? Params01[1].y : r1.w;
	r2.x = Params01[1].x < r1.w;
	r1.w = r2.x ? Params01[1].x : r1.w;
	r0.xyz = r1.xyz + r0.xyz;
	r0.xyz = r0.xyz * r1.w;
	r1.xyz = r0.xyz + r0.xyz;
	r2.xyz = r0.xyz * 0.3 + 0.05;
	r3.xy = float2(0.2, 3.333333) * Params01[1].w;
	r2.xyz = r1.xyz * r2.xyz + r3.x;
	r0.xyz = r0.xyz * 0.3 + 0.5;
	r0.xyz = r1.xyz * r0.xyz + 0.06;
	r0.xyz = r2.xyz / r0.xyz;
	r0.xyz = -Params01[1].w * 3.333333 + r0.xyz;
	r1.x = Params01[1].w * 0.2 + 19.376;
	r1.x = r1.x * 0.0408564 - r3.y;
	r1.xyz = r0.xyz / r1.x;
	r0.x = dot(r1.xyz, float3(0.2125, 0.7154, 0.0721));
	r1.xyz = r1.xyz - r0.x;
	r1.xyz = Params01[2].x * r1.xyz + r0.x;
	r2.xyz = r0.x * Params01[3].xyz - r1.xyz;
	r1.xyz = Params01[3].w * r2.xyz + r1.xyz;
	r1.xyz = Params01[2].w * r1.xyz - r0.w;
	r0.xyz = Params01[2].z * r1.xyz + r0.w;
	//last color filter used only for certain conditions, like rifle night scope
	res.xyz = lerp(r0.xyz, Params01[5].xyz, Params01[5].w);

	res.xyz = pow(res.xyz, 1.0/2.2);
	res.w=1.0;
	return res;
}

technique11 ORIGINALPOSTPROCESS <string UIName="Vanilla";> //do not modify this technique
{
	pass p0
	{
		SetVertexShader(CompileShader(vs_5_0, VS_Draw()));
		SetPixelShader(CompileShader(ps_5_0, PS_DrawOriginal()));
	}
}

